@@ -55,6 +55,7 @@ const (
 	externalName     = Prefix + "external"
 	ignoreName       = Prefix + "ignore"
 	removeName       = Prefix + "remove"
+	scriptsDirName   = Prefix + "scripts"
 	templatesDirName = Prefix + "templates"
 	versionName      = Prefix + "version"
 )
@@ -179,3 +180,11 @@ func mustTrimSuffix(s, suffix string) string {
 	}
 	return s[:len(s)-len(suffix)]
 }
+
+// ensureSuffix adds suffix to s if s is not suffixed by suffix.
+func ensureSuffix(s, suffix string) string {
+	if strings.HasSuffix(s, suffix) {
+		return s
+	}
+	return s + suffix
+}
