@@ -30,6 +30,8 @@ import (
 	gogit "github.com/go-git/go-git/v5"
 	"github.com/go-git/go-git/v5/plumbing/format/diff"
 	"github.com/google/gops/agent"
+	"github.com/gregjones/httpcache"
+	"github.com/gregjones/httpcache/diskcache"
 	"github.com/mitchellh/mapstructure"
 	"github.com/rs/zerolog"
 	"github.com/rs/zerolog/log"
@@ -80,6 +82,7 @@ type Config struct {
 	Umask              fs.FileMode                     `mapstructure:"umask"`
 	UseBuiltinAge      autoBool                        `mapstructure:"useBuiltinAge"`
 	UseBuiltinGit      autoBool                        `mapstructure:"useBuiltinGit"`
+	Verbose            bool                            `mapstructure:"verbose"`
 	WorkingTreeAbsPath chezmoi.AbsPath                 `mapstructure:"workingTree"`
 
 	// Global configuration, not settable in the config file.
@@ -96,7 +99,6 @@ type Config struct {
 	outputAbsPath    chezmoi.AbsPath
 	refreshExternals bool
 	sourcePath       bool
-	verbose          bool
 	templateFuncs    template.FuncMap
 
 	// Password manager configurations, settable in the config file.
@@ -304,6 +306,7 @@ func newConfig(options ...configOption) (*Config, error) {
 			MaxWidth: 80,
 		},
 		Edit: editCmdConfig{
+			Hardlink:    true,
 			MinDuration: 1 * time.Second,
 			exclude:     chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
 			include: chezmoi.NewEntryTypeSet(
@@ -370,6 +373,9 @@ func newConfig(options ...configOption) (*Config, error) {
 			dump: stateDumpCmdConfig{
 				format: defaultWriteDataFormat,
 			},
+			getBucket: stateGetBucketCmdConfig{
+				format: defaultWriteDataFormat,
+			},
 		},
 		status: statusCmdConfig{
 			exclude:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
@@ -395,7 +401,6 @@ func newConfig(options ...configOption) (*Config, error) {
 		// Configuration.
 		fileSystem: vfs.OSFS,
 		bds:        bds,
-		httpClient: newCacheHTTPClient(cacheDirAbsPath.Join(httpCacheDirRelPath)),
 
 		// Computed configuration.
 		homeDirAbsPath: homeDirAbsPath,
@@ -413,6 +418,7 @@ func newConfig(options ...configOption) (*Config, error) {
 		"bitwardenFields":          c.bitwardenFieldsTemplateFunc,
 		"decrypt":                  c.decryptTemplateFunc,
 		"encrypt":                  c.encryptTemplateFunc,
+		"fromYaml":                 c.fromYamlTemplateFunc,
 		"gitHubKeys":               c.gitHubKeysTemplateFunc,
 		"gitHubLatestRelease":      c.gitHubLatestReleaseTemplateFunc,
 		"gopass":                   c.gopassTemplateFunc,
@@ -437,6 +443,7 @@ func newConfig(options ...configOption) (*Config, error) {
 		"secret":                   c.secretTemplateFunc,
 		"secretJSON":               c.secretJSONTemplateFunc,
 		"stat":                     c.statTemplateFunc,
+		"toYaml":                   c.toYamlTemplateFunc,
 		"vault":                    c.vaultTemplateFunc,
 	} {
 		c.addTemplateFunc(key, value)
@@ -1114,9 +1121,14 @@ func (c *Config) filterInput(args []string, f func([]byte) ([]byte, error)) erro
 // findFirstConfigTemplate searches for a config template, returning the path,
 // format, and contents of the first one that it finds.
 func (c *Config) findFirstConfigTemplate() (chezmoi.RelPath, string, []byte, error) {
+	sourceDirAbsPath, err := c.sourceDirAbsPath()
+	if err != nil {
+		return chezmoi.EmptyRelPath, "", nil, err
+	}
+
 	for _, ext := range viper.SupportedExts {
 		filename := chezmoi.NewRelPath(chezmoi.Prefix + "." + ext + chezmoi.TemplateSuffix)
-		contents, err := c.baseSystem.ReadFile(c.SourceDirAbsPath.Join(filename))
+		contents, err := c.baseSystem.ReadFile(sourceDirAbsPath.Join(filename))
 		switch {
 		case errors.Is(err, fs.ErrNotExist):
 			continue
@@ -1128,6 +1140,22 @@ func (c *Config) findFirstConfigTemplate() (chezmoi.RelPath, string, []byte, err
 	return chezmoi.EmptyRelPath, "", nil, nil
 }
 
+func (c *Config) getHTTPClient() (*http.Client, error) {
+	if c.httpClient != nil {
+		return c.httpClient, nil
+	}
+
+	httpCacheBasePath, err := c.baseSystem.RawPath(c.CacheDirAbsPath.Join(httpCacheDirRelPath))
+	if err != nil {
+		return nil, err
+	}
+	httpCache := diskcache.New(httpCacheBasePath.String())
+	httpTransport := httpcache.NewTransport(httpCache)
+	c.httpClient = httpTransport.Client()
+
+	return c.httpClient, nil
+}
+
 // gitAutoAdd adds all changes to the git index and returns the new git status.
 func (c *Config) gitAutoAdd() (*git.Status, error) {
 	if err := c.run(c.WorkingTreeAbsPath, c.Git.Command, []string{"add", "."}); err != nil {
@@ -1223,6 +1251,7 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 	persistentFlags.VarP(&c.SourceDirAbsPath, "source", "S", "Set source directory")
 	persistentFlags.Var(&c.UseBuiltinAge, "use-builtin-age", "Use builtin age")
 	persistentFlags.Var(&c.UseBuiltinGit, "use-builtin-git", "Use builtin git")
+	persistentFlags.BoolVarP(&c.Verbose, "verbose", "v", c.Verbose, "Make output more verbose")
 	persistentFlags.VarP(&c.WorkingTreeAbsPath, "working-tree", "W", "Set working tree directory")
 	for viperKey, key := range map[string]string{
 		"color":           "color",
@@ -1233,6 +1262,7 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 		"sourceDir":       "source",
 		"useBuiltinAge":   "use-builtin-age",
 		"useBuiltinGit":   "use-builtin-git",
+		"verbose":         "verbose",
 		"workingTree":     "working-tree",
 	} {
 		if err := viper.BindPFlag(viperKey, persistentFlags.Lookup(key)); err != nil {
@@ -1255,7 +1285,6 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 	persistentFlags.VarP(&c.outputAbsPath, "output", "o", "Write output to path instead of stdout")
 	persistentFlags.BoolVarP(&c.refreshExternals, "refresh-externals", "R", c.refreshExternals, "Refresh external cache")
 	persistentFlags.BoolVar(&c.sourcePath, "source-path", c.sourcePath, "Specify targets by source path")
-	persistentFlags.BoolVarP(&c.verbose, "verbose", "v", c.verbose, "Make output more verbose")
 
 	for _, err := range []error{
 		rootCmd.MarkPersistentFlagFilename("config"),
@@ -1323,15 +1352,16 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 func (c *Config) newSourceState(
 	ctx context.Context, options ...chezmoi.SourceStateOption,
 ) (*chezmoi.SourceState, error) {
+	httpClient, err := c.getHTTPClient()
+	if err != nil {
+		return nil, err
+	}
+
 	sourceStateLogger := c.logger.With().Str(logComponentKey, logComponentValueSourceState).Logger()
 
-	sourceDirAbsPath := c.SourceDirAbsPath
-	switch data, err := c.sourceSystem.ReadFile(c.SourceDirAbsPath.JoinString(chezmoi.RootName)); {
-	case errors.Is(err, fs.ErrNotExist):
-	case err != nil:
+	sourceDirAbsPath, err := c.sourceDirAbsPath()
+	if err != nil {
 		return nil, err
-	default:
-		sourceDirAbsPath = c.SourceDirAbsPath.JoinString(string(bytes.TrimSpace(data)))
 	}
 
 	s := chezmoi.NewSourceState(append([]chezmoi.SourceStateOption{
@@ -1340,7 +1370,7 @@ func (c *Config) newSourceState(
 		chezmoi.WithDefaultTemplateDataFunc(c.defaultTemplateData),
 		chezmoi.WithDestDir(c.DestDirAbsPath),
 		chezmoi.WithEncryption(c.encryption),
-		chezmoi.WithHTTPClient(c.httpClient),
+		chezmoi.WithHTTPClient(httpClient),
 		chezmoi.WithInterpreters(c.Interpreters),
 		chezmoi.WithLogger(&sourceStateLogger),
 		chezmoi.WithMode(c.Mode),
@@ -1565,12 +1595,14 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 		c.sourceSystem = chezmoi.NewDryRunSystem(c.sourceSystem)
 		c.destSystem = chezmoi.NewDryRunSystem(c.destSystem)
 	}
-	if c.verbose {
+	if c.Verbose {
 		c.sourceSystem = chezmoi.NewGitDiffSystem(c.sourceSystem, c.stdout, c.SourceDirAbsPath, &chezmoi.GitDiffSystemOptions{
-			Color: color,
+			Color:   color,
+			Include: c.Diff.include.Sub(c.Diff.Exclude),
 		})
 		c.destSystem = chezmoi.NewGitDiffSystem(c.destSystem, c.stdout, c.DestDirAbsPath, &chezmoi.GitDiffSystemOptions{
-			Color: color,
+			Color:   color,
+			Include: c.Diff.include.Sub(c.Diff.Exclude),
 		})
 	}
 
@@ -1789,6 +1821,19 @@ func (c *Config) sourceAbsPaths(sourceState *chezmoi.SourceState, args []string)
 	return sourceAbsPaths, nil
 }
 
+// sourceDirAbsPath returns the source directory, using .chezmoiroot if it
+// exists.
+func (c *Config) sourceDirAbsPath() (chezmoi.AbsPath, error) {
+	switch data, err := c.sourceSystem.ReadFile(c.SourceDirAbsPath.JoinString(chezmoi.RootName)); {
+	case errors.Is(err, fs.ErrNotExist):
+		return c.SourceDirAbsPath, nil
+	case err != nil:
+		return chezmoi.EmptyAbsPath, err
+	default:
+		return c.SourceDirAbsPath.JoinString(string(bytes.TrimSpace(data))), nil
+	}
+}
+
 type targetRelPathsOptions struct {
 	mustBeInSourceState bool
 	recursive           bool
