@@ -11,11 +11,13 @@ import (
 )
 
 type stateCmdConfig struct {
-	data   stateDataCmdConfig
-	delete stateDeleteCmdConfig
-	dump   stateDumpCmdConfig
-	get    stateGetCmdConfig
-	set    stateSetCmdConfig
+	data         stateDataCmdConfig
+	delete       stateDeleteCmdConfig
+	deleteBucket stateDeleteBucketCmdConfig
+	dump         stateDumpCmdConfig
+	get          stateGetCmdConfig
+	getBucket    stateGetBucketCmdConfig
+	set          stateSetCmdConfig
 }
 
 type stateDataCmdConfig struct {
@@ -27,6 +29,10 @@ type stateDeleteCmdConfig struct {
 	key    string
 }
 
+type stateDeleteBucketCmdConfig struct {
+	bucket string
+}
+
 type stateDumpCmdConfig struct {
 	format writeDataFormat
 }
@@ -36,6 +42,11 @@ type stateGetCmdConfig struct {
 	key    string
 }
 
+type stateGetBucketCmdConfig struct {
+	bucket string
+	format writeDataFormat
+}
+
 type stateSetCmdConfig struct {
 	bucket string
 	key    string
@@ -77,6 +88,21 @@ func (c *Config) newStateCmd() *cobra.Command {
 	stateDeletePersistentFlags.StringVar(&c.state.delete.key, "key", c.state.delete.key, "key")
 	stateCmd.AddCommand(stateDeleteCmd)
 
+	stateDeleteBucketCmd := &cobra.Command{
+		Use:   "delete-bucket",
+		Short: "Delete a bucket from the persistent state",
+		Args:  cobra.NoArgs,
+		RunE:  c.runStateDeleteBucketCmd,
+		Annotations: map[string]string{
+			persistentStateMode: persistentStateModeReadWrite,
+		},
+	}
+	stateDeleteBucketPersistentFlags := stateDeleteBucketCmd.PersistentFlags()
+	stateDeleteBucketPersistentFlags.StringVar(
+		&c.state.deleteBucket.bucket, "bucket", c.state.deleteBucket.bucket, "bucket",
+	)
+	stateCmd.AddCommand(stateDeleteBucketCmd)
+
 	stateDumpCmd := &cobra.Command{
 		Use:   "dump",
 		Short: "Generate a dump of the persistent state",
@@ -104,6 +130,20 @@ func (c *Config) newStateCmd() *cobra.Command {
 	stateGetPersistentFlags.StringVar(&c.state.get.key, "key", c.state.get.key, "key")
 	stateCmd.AddCommand(stateGetCmd)
 
+	stateGetBucketCmd := &cobra.Command{
+		Use:   "get-bucket",
+		Short: "Get a bucket from the persistent state",
+		Args:  cobra.NoArgs,
+		RunE:  c.runStateGetBucketCmd,
+		Annotations: map[string]string{
+			persistentStateMode: persistentStateModeReadOnly,
+		},
+	}
+	stateGetBucketPersistentFlags := stateGetBucketCmd.PersistentFlags()
+	stateGetBucketPersistentFlags.StringVar(&c.state.getBucket.bucket, "bucket", c.state.getBucket.bucket, "bucket")
+	stateGetBucketPersistentFlags.VarP(&c.state.getBucket.format, "format", "f", "format")
+	stateCmd.AddCommand(stateGetBucketCmd)
+
 	stateResetCmd := &cobra.Command{
 		Use:   "reset",
 		Short: "Reset the persistent state",
@@ -145,6 +185,10 @@ func (c *Config) runStateDeleteCmd(cmd *cobra.Command, args []string) error {
 	return c.persistentState.Delete([]byte(c.state.delete.bucket), []byte(c.state.delete.key))
 }
 
+func (c *Config) runStateDeleteBucketCmd(cmd *cobra.Command, args []string) error {
+	return c.persistentState.DeleteBucket([]byte(c.state.deleteBucket.bucket))
+}
+
 func (c *Config) runStateDumpCmd(cmd *cobra.Command, args []string) error {
 	data, err := chezmoi.PersistentStateData(c.persistentState)
 	if err != nil {
@@ -161,6 +205,14 @@ func (c *Config) runStateGetCmd(cmd *cobra.Command, args []string) error {
 	return c.writeOutput(value)
 }
 
+func (c *Config) runStateGetBucketCmd(cmd *cobra.Command, args []string) error {
+	data, err := chezmoi.PersistentStateBucketData(c.persistentState, []byte(c.state.getBucket.bucket))
+	if err != nil {
+		return err
+	}
+	return c.marshal(c.state.getBucket.format, data)
+}
+
 func (c *Config) runStateResetCmd(cmd *cobra.Command, args []string) error {
 	persistentStateFileAbsPath, err := c.persistentStateFile()
 	if err != nil {
